package jetbrains.mps.gtext.runtime;

/*Generated by MPS */

import java.util.Map;
import java.util.Stack;
import java.util.List;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.logging.Logger;
import java.util.TreeMap;

public class TBaseBuilderContext {
  protected static String ROOT = "__ROOT__";
  protected Map<String, TContent> myContents;
  protected Stack<TContent> myContentsStack;
  protected TBuffer myBuffer;
  protected List<TBaseBuilderContextListener> myListeners;

  public TBaseBuilderContext() {
  }

  public void initBuffer(TBuffer buffer) {
    myContents = null;
    myContentsStack = null;
    TContent rootContent = new TContent(ROOT, buffer);
    getContents().put(ROOT, rootContent);
    rootContent.setPosition(0);
    getContentsStack().push(getContents().get(ROOT));
    myBuffer = buffer;
  }

  public void initBuffer() {
    initBuffer(new TBuffer());
  }

  public String getText() {
    if (getContentsStack().size() != 1) {
      throw new IllegalStateException("Can't get text, because there is open content block [" + getContentsStack().peek().getName() + "]");
    }
    notify(new TBaseBuilderContext.ListenerVisitor() {
      @Override
      public void visit(TBaseBuilderContextListener l) {
        l.getTextCalled(TBaseBuilderContext.this);
      }
    });
    if (getContents().size() == 1) {
      return myBuffer.getText();
    }
    List<TContent> sortedContents = new ArrayList<TContent>(getContents().values());
    sortedContents.remove(getContentsStack().peek());
    Collections.sort(sortedContents, new Comparator<TContent>() {
      @Override
      public int compare(TContent o1, TContent o2) {
        return o2.getPosition() - o1.getPosition();
      }
    });
    for (TContent c : sortedContents) {
      if (c.getPosition() == -1) {
        Logger.getLogger(TBaseBuilderContext.class.getName()).info("There is no placeholder for content [" + c.getName() + "]");
      } else {
        myBuffer.getStringBuilder().insert(c.getPosition(), c.getBuf().getText());
      }
    }
    return myBuffer.getText();
  }

  public void append(String text) {
    myBuffer.append(text);
  }

  public void appendIndent() {
    myBuffer.appendIndent();
  }

  public void appendNewLine() {
    myBuffer.appendNewLine();
  }

  public void increaseIndent() {
    myBuffer.increaseIndent();
  }

  public void decreaseIndent() {
    myBuffer.decreaseIndent();
  }

  protected Map<String, TContent> getContents() {
    if (myContents == null) {
      myContents = new TreeMap<String, TContent>();
    }
    return myContents;
  }

  protected Stack<TContent> getContentsStack() {
    if (myContentsStack == null) {
      myContentsStack = new Stack<TContent>();
    }
    return myContentsStack;
  }

  protected int getCurrentPosition() {
    return getContents().get(ROOT).getBuf().getCurrentPosition();
  }

  public void addContentPlaceholder(String name) {
    TContent c = getContentBlock(name);
    c.setPosition(getCurrentPosition());
  }

  public void startContentBlock(String name) {
    TContent c = getContentBlock(name);
    getContentsStack().push(c);
    myBuffer = c.getBuf();
  }

  private TContent getContentBlock(String name) {
    TContent content = getContents().get(name);
    if (content == null) {
      content = new TContent(name, new TBuffer());
      getContents().put(name, content);
    }
    return content;
  }

  public void endContentBlock() {
    getContentsStack().pop();
    myBuffer = getContentsStack().peek().getBuf();
  }

  public TContent getCurrentContent() {
    return getContentsStack().peek();
  }

  private List<TBaseBuilderContextListener> getListeners() {
    if (myListeners == null) {
      myListeners = new ArrayList<TBaseBuilderContextListener>();
    }
    return myListeners;
  }

  public void addListener(TBaseBuilderContextListener l) {
    getListeners().add(l);
  }

  public void removeListener(TBaseBuilderContextListener l) {
    getListeners().remove(l);
  }

  protected void notify(TBaseBuilderContext.ListenerVisitor v) {
    if (myListeners == null) {
      return;
    }
    for (TBaseBuilderContextListener l : myListeners) {
      v.visit(l);
    }
  }

  protected static interface ListenerVisitor {
    public void visit(TBaseBuilderContextListener l);
  }
}
